SHORT_BLOCK_WINDOW = 20
"""Max. number of blocks for a short-term order to live. See [source](https://docs.dydx.xyz/concepts/trading/orders#comparison)"""

STATEFUL_ORDER_TIME_WINDOW = 95*3600*24
"""Max. time window for a stateful order to live. See [source](https://docs.dydx.xyz/concepts/trading/orders#comparison)"""